<#
.SYNOPSIS
  Fast, chunked batch processing of Corona .cxr files via CoronaImageCmd.exe

.DESCRIPTION
  Splits all *.cxr in the current directory into batches and calls CoronaImageCmd.exe
  once per batch, preserving high throughput while avoiding Windows' ~8 KB command-line limit.

.PARAMETER InputElement
  Which element to process (default: '*').

.PARAMETER OutputFormat
  Output image format (png, bmp, jpg, exr, cxr).

.PARAMETER PostprocessConfig
  Path to your Postprocess.conf (if present).

.PARAMETER LightMixConfig
  Path to your LightMix.conf (if present).

.PARAMETER BatchSize
  How many files per CoronaImageCmd.exe invocation (tune this for your path lengths).

.EXAMPLE
  # Run with defaults:
  .\Process-CoronaImages.ps1

  # If you have very long paths, you might lower batch size:
  .\Process-CoronaImages.ps1 -BatchSize 50
#>

param(
    [string]$InputElement    = '*',
    [string]$OutputFormat    = 'png',
    [string]$PostprocessConfig = 'Postprocess.conf',
    [string]$LightMixConfig  = 'LightMix.conf',
    [int]   $BatchSize       = 100
)

# Locate the CoronaImageCmd.exe (adjust if your install is elsewhere)
$editorExecutable = Join-Path $Env:ProgramFiles `
    'Chaos\Corona\Corona Renderer for 3ds Max\Image Editor\CoronaImageCmd.exe'

# Build the static portion of the arguments
$commonArgs = @('--batch')
if (Test-Path $PostprocessConfig) {
    $commonArgs += '--config'; $commonArgs += $PostprocessConfig
}
if (Test-Path $LightMixConfig) {
    $commonArgs += '--config'; $commonArgs += $LightMixConfig
}
$commonArgs += '--element'; $commonArgs += $InputElement

Write-Host "Starting batch processing at $(Get-Date -Format 'HH:mm:ss')" -ForegroundColor Cyan

# Grab all .cxr files in this directory
$files = Get-ChildItem -File -Filter '*.cxr' | Sort-Object Name

# Process in chunks
for ($i = 0; $i -lt $files.Count; $i += $BatchSize) {
    # Determine the slice of files for this batch
    $sliceEnd = [Math]::Min($i + $BatchSize - 1, $files.Count - 1)
    $batch = $files[$i..$sliceEnd]

    # Build full argument list: commonArgs + [in1,out1,in2,out2,...]
    $args = @($commonArgs)
    foreach ($f in $batch) {
        $inPath  = $f.FullName
        $outName = "Post_$($f.BaseName).$OutputFormat"
        $outPath = Join-Path $f.DirectoryName $outName
        $args += $inPath; $args += $outPath
    }

    Write-Host " Processing batch $([int]($i/$BatchSize)+1) of $([Math]::Ceiling($files.Count/$BatchSize))" `
        "($($batch.Count) files) at $(Get-Date -Format 'HH:mm:ss')..."

    # Invoke the editor with our args array
    & $editorExecutable @args
}

Write-Host "Finished at $(Get-Date -Format 'HH:mm:ss')" -ForegroundColor Cyan
